SWDEV-326631 - Adds sample which tests HIP language support with upstream CMake (#2743)
Change-Id: I6b666cc0c5b1a53819515c5e343e517a2712d2ef
[ROCm/hip commit: a49143593c]
このコミットが含まれているのは:
@@ -0,0 +1,27 @@
|
|||||||
|
# Copyright (C) 2022 Advanced Micro Devices, Inc. All Rights Reserved.
|
||||||
|
#
|
||||||
|
# Permission is hereby granted, free of charge, to any person obtaining a copy
|
||||||
|
# of this software and associated documentation files (the "Software"), to deal
|
||||||
|
# in the Software without restriction, including without limitation the rights
|
||||||
|
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
||||||
|
# copies of the Software, and to permit persons to whom the Software is
|
||||||
|
# furnished to do so, subject to the following conditions:
|
||||||
|
#
|
||||||
|
# The above copyright notice and this permission notice shall be included in
|
||||||
|
# all copies or substantial portions of the Software.
|
||||||
|
#
|
||||||
|
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
||||||
|
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
||||||
|
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
||||||
|
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
||||||
|
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
||||||
|
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
||||||
|
# THE SOFTWARE.
|
||||||
|
|
||||||
|
cmake_minimum_required(VERSION 3.21.3)
|
||||||
|
|
||||||
|
project(cmake_hip_lang_support VERSION 1.0
|
||||||
|
DESCRIPTION "HIP Language Support with upstream CMake"
|
||||||
|
LANGUAGES HIP)
|
||||||
|
# Create the executable
|
||||||
|
add_executable(square square.hip)
|
||||||
@@ -0,0 +1,16 @@
|
|||||||
|
### This will test HIP language support in upstream CMake
|
||||||
|
I. Build
|
||||||
|
mkdir -p build; cd build
|
||||||
|
rm -rf *; cmake ..
|
||||||
|
make
|
||||||
|
|
||||||
|
II. Test
|
||||||
|
$ ./square
|
||||||
|
info: running on device
|
||||||
|
info: allocate host mem ( 7.63 MB)
|
||||||
|
info: allocate device mem ( 7.63 MB)
|
||||||
|
info: copy Host2Device
|
||||||
|
info: launch 'vector_square' kernel
|
||||||
|
info: copy Device2Host
|
||||||
|
info: check result
|
||||||
|
PASSED!
|
||||||
@@ -0,0 +1,98 @@
|
|||||||
|
/*
|
||||||
|
Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
|
||||||
|
|
||||||
|
Permission is hereby granted, free of charge, to any person obtaining a copy
|
||||||
|
of this software and associated documentation files (the "Software"), to deal
|
||||||
|
in the Software without restriction, including without limitation the rights
|
||||||
|
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
||||||
|
copies of the Software, and to permit persons to whom the Software is
|
||||||
|
furnished to do so, subject to the following conditions:
|
||||||
|
|
||||||
|
The above copyright notice and this permission notice shall be included in
|
||||||
|
all copies or substantial portions of the Software.
|
||||||
|
|
||||||
|
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
||||||
|
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
||||||
|
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
||||||
|
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
||||||
|
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
||||||
|
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
||||||
|
THE SOFTWARE.
|
||||||
|
*/
|
||||||
|
|
||||||
|
#include <stdio.h>
|
||||||
|
#include <hip/hip_runtime.h>
|
||||||
|
|
||||||
|
#define CHECK(cmd) \
|
||||||
|
{\
|
||||||
|
hipError_t error = cmd;\
|
||||||
|
if (error != hipSuccess) { \
|
||||||
|
fprintf(stderr, "error: '%s'(%d) at %s:%d\n", hipGetErrorString(error), error,__FILE__, __LINE__); \
|
||||||
|
exit(EXIT_FAILURE);\
|
||||||
|
}\
|
||||||
|
}
|
||||||
|
|
||||||
|
|
||||||
|
/*
|
||||||
|
* Square each element in the array A and write to array C.
|
||||||
|
*/
|
||||||
|
template <typename T>
|
||||||
|
__global__ void
|
||||||
|
vector_square(T *C_d, T *A_d, size_t N)
|
||||||
|
{
|
||||||
|
size_t offset = (blockIdx.x * blockDim.x + threadIdx.x);
|
||||||
|
size_t stride = blockDim.x * gridDim.x ;
|
||||||
|
|
||||||
|
for (size_t i=offset; i<N; i+=stride) {
|
||||||
|
C_d[i] = A_d[i] * A_d[i];
|
||||||
|
}
|
||||||
|
}
|
||||||
|
|
||||||
|
|
||||||
|
int main(int argc, char *argv[])
|
||||||
|
{
|
||||||
|
float *A_d, *C_d;
|
||||||
|
float *A_h, *C_h;
|
||||||
|
size_t N = 1000000;
|
||||||
|
size_t Nbytes = N * sizeof(float);
|
||||||
|
|
||||||
|
hipDeviceProp_t props;
|
||||||
|
CHECK(hipGetDeviceProperties(&props, 0/*deviceID*/));
|
||||||
|
printf ("info: running on device %s\n", props.name);
|
||||||
|
|
||||||
|
printf ("info: allocate host mem (%6.2f MB)\n", 2*Nbytes/1024.0/1024.0);
|
||||||
|
A_h = (float*)malloc(Nbytes);
|
||||||
|
CHECK(A_h == 0 ? hipErrorOutOfMemory : hipSuccess );
|
||||||
|
C_h = (float*)malloc(Nbytes);
|
||||||
|
CHECK(C_h == 0 ? hipErrorOutOfMemory : hipSuccess );
|
||||||
|
// Fill with Phi + i
|
||||||
|
for (size_t i=0; i<N; i++)
|
||||||
|
{
|
||||||
|
A_h[i] = 1.618f + i;
|
||||||
|
}
|
||||||
|
|
||||||
|
printf ("info: allocate device mem (%6.2f MB)\n", 2*Nbytes/1024.0/1024.0);
|
||||||
|
CHECK(hipMalloc(&A_d, Nbytes));
|
||||||
|
CHECK(hipMalloc(&C_d, Nbytes));
|
||||||
|
|
||||||
|
|
||||||
|
printf ("info: copy Host2Device\n");
|
||||||
|
CHECK ( hipMemcpy(A_d, A_h, Nbytes, hipMemcpyHostToDevice));
|
||||||
|
|
||||||
|
const unsigned blocks = 512;
|
||||||
|
const unsigned threadsPerBlock = 256;
|
||||||
|
|
||||||
|
printf ("info: launch 'vector_square' kernel\n");
|
||||||
|
hipLaunchKernelGGL(vector_square, dim3(blocks), dim3(threadsPerBlock), 0, 0, C_d, A_d, N);
|
||||||
|
|
||||||
|
printf ("info: copy Device2Host\n");
|
||||||
|
CHECK ( hipMemcpy(C_h, C_d, Nbytes, hipMemcpyDeviceToHost));
|
||||||
|
|
||||||
|
printf ("info: check result\n");
|
||||||
|
for (size_t i=0; i<N; i++) {
|
||||||
|
if (C_h[i] != A_h[i] * A_h[i]) {
|
||||||
|
CHECK(hipErrorUnknown);
|
||||||
|
}
|
||||||
|
}
|
||||||
|
printf ("PASSED!\n");
|
||||||
|
}
|
||||||
新しいイシューから参照
ユーザーをブロックする